import os
from datetime import datetime
import sqlite3
from contextlib import closing

class WeatherDb:
    def __init__(self, filename, locations):
        path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        self.db = os.path.join(path, filename)
        self.createSchema()
        self.insertLocations(locations)

    def dict_factory(self, cursor, row):
        d = {}
        for idx, col in enumerate(cursor.description):
            d[col[0]] = row[idx]
        return d

    def execute(self, sql, parameters=(), results=lambda c: c.fetchall()):
        with sqlite3.connect(self.db, detect_types=sqlite3.PARSE_DECLTYPES) as con:
            con.row_factory = self.dict_factory
            with closing(con.cursor()) as c:
                c.execute(sql, parameters)
                result = results(c)
                con.commit()
        return result

    def createSchema(self):
        self.execute("""
CREATE TABLE IF NOT EXISTS locations (
    id INTEGER PRIMARY KEY,
    name TEXT NOT NULL UNIQUE
)
""")

        self.execute("""
CREATE TABLE IF NOT EXISTS readings (
    date TIMESTAMP NOT NULL,
    location INTEGER NOT NULL,
    temperature REAL,
    pressure REAL,
    humidity REAL,
    CONSTRAINT PK_readings PRIMARY KEY (location, date)
)
""")

    def insertLocations(self, locations):
        for location in locations:
            self.execute("""
INSERT OR IGNORE INTO locations (name) VALUES (?)
""", (location,))

    def insertReading(self, date, location, temperature, pressure, humidity):
        self.execute("""
INSERT INTO readings (
    date,
    location,
    temperature,
    pressure,
    humidity)
VALUES (?, (SELECT id FROM locations WHERE name LIKE ?), ?, ?, ?)
""", (date, location, temperature, pressure, humidity))

    def getReadings(self, location, fromDate, toDate):
        return self.execute("""
SELECT
    date,
    :location AS location,
    temperature,
    pressure,
    humidity
FROM
    readings
WHERE
    location = (SELECT id FROM locations WHERE name LIKE :location)
    AND date >= :fromDate
    AND date < :toDate
ORDER BY
    date DESC
""", {"location": location, "fromDate": fromDate, "toDate": toDate})

    def getReadingsAggregate(self, location, fromDate, toDate, groupBy):
        groupings = {
            "hour": "%Y-%m-%dT%H:00+00:00",
            "day": "%Y-%m-%dT00:00+00:00"
        }
        return self.execute("""
SELECT
    strftime(:grouping, date) AS date,
    :location AS location,
    AVG(temperature) AS temperature,
    AVG(pressure) AS pressure,
    AVG(humidity) AS humidity
FROM
    readings
WHERE
    location = (SELECT id FROM locations WHERE name LIKE :location)
    AND date >= :fromDate
    AND date < :toDate
GROUP BY
    strftime(:grouping, date)
ORDER BY
    date DESC
""", {"location": location, "fromDate": fromDate, "toDate": toDate, "grouping": groupings[groupBy]})

    def deleteReadings(self, fromDate, toDate):
        self.execute("""
DELETE FROM
    readings
WHERE
    (:fromDate IS NULL OR date >= :fromDate)
    AND (:toDate IS NULL OR date < :toDate)
""", {"fromDate": fromDate, "toDate": toDate})

    def getMostRecentReadingTime(self):
        return self.execute("""
SELECT
    MAX(date)
FROM
    readings
""", (), lambda c: c.fetchone()["date"] if c.rowcount > 0 else None)
