import webiopi
from webiopi.devices.instance import DEVICES
from threading import Lock
from datetime import datetime, date, timezone
import json
import sys
import os
import subprocess
sys.path.append(os.path.dirname(__file__))
from db import WeatherDb

#Save a reading to the DB every this many seconds.
RECORDING_PERIOD = 60.0
#Consecutive readings fewer than this many seconds apart will return the same value.
CACHE_PERIOD = 1.0

lastRecordingTime = None
lastReading = {}
sensors = {}
db = None
sensorLock = Lock()

#webiopi.setDebug()

# setup function is automatically called at WebIOPi startup
def setup():
    global lastRecordingTime
    global sensors
    global db

    for name in DEVICES:
        device = webiopi.deviceInstance(name)
        #TODO Is it worth also checking that the device has the correct 3 sensor types?
        if hasattr(device, "__getAll__"):
            sensors[name] = device
            lastReading[name] = None

    db = WeatherDb("weather.db", list(sensors.keys()))
    lastRecordingTime = db.getMostRecentReadingTime()

# loop function is repeatedly called by WebIOPi
def loop():
    global lastRecordingTime

    if lastRecordingTime == None or (datetime.now(timezone.utc) - lastRecordingTime).total_seconds() >= RECORDING_PERIOD:
        lastRecordingTime = datetime.now(timezone.utc)
        for location in sensors:
            reading = readFromSensors(location)
            db.insertReading(reading["date"], reading["location"], reading["temperature"], reading["pressure"], reading["humidity"])

    webiopi.sleep(1)

# destroy function is called at WebIOPi shutdown
def destroy():
    pass

def readFromSensors(location):
    global lastReading

    with sensorLock:
        if lastReading[location] == None or (datetime.now(timezone.utc) - lastReading[location]["date"]).total_seconds() > CACHE_PERIOD:
            sensor = sensors[location]
            (T, P ,H) = sensor.__getAll__()

            lastReading[location] = {
                "location": location,
                "date": datetime.now(timezone.utc),
                "temperature": T,
                "pressure": P,
                "humidity": H
            }

    return lastReading[location]

@webiopi.macro('GET', 'locations', 'application/json')
def getLocations():
    #Want to have sensors in a consistent order, and I2C address is as good as anything to sort by.
    return json.dumps(list(sorted(sensors.keys(), key=lambda location: sensors[location].slave)))

@webiopi.macro('GET', 'readings', 'application/json')
def getReadings(path, fromDate=None, toDate=None, groupBy=None):
    location = path
    current = fromDate == None and toDate == None
    # TODO: Date parsing with strptime won't parse all ISO8601 formats.
    readings = None
    if current:
        readings = readFromSensors(location)
    elif not groupBy:
        readings = db.getReadings(location, datetime.strptime(fromDate, "%Y-%m-%dT%H:%M:%S.%fZ"), datetime.strptime(toDate, "%Y-%m-%dT%H:%M:%S.%fZ"))
    else:
        readings = db.getReadingsAggregate(location, datetime.strptime(fromDate, "%Y-%m-%dT%H:%M:%S.%fZ"), datetime.strptime(toDate, "%Y-%m-%dT%H:%M:%S.%fZ"), groupBy)
    return json.dumps(readings, default=lambda obj: (
        obj.replace(tzinfo=timezone.utc).isoformat()
        if isinstance(obj, datetime)
        or isinstance(obj, date)
        else None
    ))

@webiopi.macro('POST', 'display', 'text/plain')
def setDisplay(enable):
    status = "on" if enable == "true" else "off"
    #This has to be run as a user with access to the display, like the default "pi" user.
    subprocess.check_output(["sudo", "-H", "-u", "pi", "bash", "-c", "xset -display :0 dpms force " + status])

#@webiopi.macro('POST', 'leds', 'text/plain')
#def setLeds(enable):
#    #RPi3 only allows ACT LED to be controlled, not the PWR LED.
#    status = "1" if enable == "true" else "0"
#    p = subprocess.Popen(["sudo", "tee", "/sys/class/leds/led0/brightness"], stdin=subprocess.PIPE)
#    output = p.communicate(bytes(status, 'UTF-8'))
#    return p.returncode
